package meerkat.voting.gui.select_candidate_by_picture;

import com.google.protobuf.ByteString;
import javafx.embed.swing.SwingFXUtils;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.control.CheckBox;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;
import meerkat.protobuf.BallotQuestionUIElementOuterClass;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.IOException;


/**
 * Created by Vladimir Eliezer Tokarev on 10/3/2016.
 * This object updates the visual representations of the voters binaryDatas
 */
class PicturesAnswersUpdater implements EventHandler{
    private Stage currentStage;
    private int columIndex;

    PicturesAnswersUpdater(Stage primaryStage) {
        this.currentStage = primaryStage;
        this.columIndex = 0;
        // The lookup works only after the css have been randered
        this.currentStage.getScene().getRoot().applyCss();
    }

    /**
     * Gets all the binaryDatas from the ballot ui question and puts them into the binaryDatas container
     *
     * @param question
     */
    void UpdateAnswers(BallotQuestionUIElementOuterClass.BallotQuestionUIElement question) {
        this.RemoveAllAnswers();
        for (ByteString bytesAnswer : question.getAnswers().getAnswers().getAnswersList()) {
            try {
                this.AddAnswer(bytesAnswer);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        this.currentStage.show();
    }

    /**
     * Gets the container of the binaryDatas
     *
     * @return GridPane object
     */
    private GridPane GetAnswersContainer() {
        return (GridPane) this.currentStage.getScene().lookup("#AnswersGridPane");
    }

    /**
     * Remove all previous binaryDatas from the container
     */
    private void RemoveAllAnswers() {
        this.GetAnswersContainer().getChildren().removeAll();
    }

    /**
     * Creates binaryData element which is grid pane with the binaryData and check box
     *
     * @param binaryData string binaryData to show to the voter
     * @return GridPane which contains string and check box
     */
    private GridPane GetAnswerElement(ByteString binaryData) throws IOException {
        GridPane gridPane = new GridPane();

        BufferedImage bufferedImage = ImageIO.read(binaryData.newInput());
        Image image = SwingFXUtils.toFXImage(bufferedImage, null);

        ImageView imageView = new ImageView();
        imageView.setImage(image);

        CheckBox checkBox = new CheckBox();
        checkBox.setOnAction(this);

        gridPane.add(imageView, 2 , 1);
        gridPane.add(checkBox, 2, 2);

        gridPane.setPrefSize(100, 100);
        gridPane.setPadding(new Insets(10));

        return gridPane;
    }

    /**
     * Adds the GridPane binaryData to the panel
     *
     * @param binaryData the binaryData string to represent to the user
     */
    private void AddAnswer(ByteString binaryData) throws IOException {
        GridPane container = this.GetAnswersContainer();

        GridPane newAnswer = this.GetAnswerElement(binaryData);
        container.addColumn(1);

        this.columIndex++;
        container.add(newAnswer, this.columIndex, 0);
        this.currentStage.show();
    }

    @Override
    public void handle(Event event) {

    }
}
